"""
Select widget for MonthField. Copied and modified from
https://docs.djangoproject.com/en/1.8/ref/forms/widgets/#base-widget-classes
"""
import datetime
from datetime import date

from django.forms import widgets
from django.utils.dates import MONTHS
from .util import string_type


def calc_years():
    now = datetime.datetime.now()
    this_year = now.year
    year_span = range(2015, this_year + 2)
    _years = list(zip(year_span, year_span))
    return _years


years = calc_years()


class MonthSelectorWidget(widgets.MultiWidget):
    class Media:
        css = {
            'screen': ('month/field/widget_month.css',)
        }

    def __init__(self, attrs=None):
        # create choices for days, months, years
        _widgets = list()
        _attrs = attrs or {}  # default class
        _attrs['class'] = (_attrs.get('class', '') + ' w-month-year').strip()
        _widgets.append(widgets.Select(attrs=_attrs, choices=MONTHS.items()))
        _attrs['class'] += " w-year"
        _widgets.insert(0, widgets.Select(attrs=_attrs, choices=years))

        super(MonthSelectorWidget, self).__init__(_widgets, attrs)

    def decompress(self, value):
        if value:
            if isinstance(value, string_type):
                m = int(value[5:7])
                y = int(value[:4])
                return [y, m]
            return [value.year, value.month]
        return [None, None]

    def format_output(self, rendered_widgets):
        return ''.join(rendered_widgets)

    def value_from_datadict(self, data, files, name):
        datelist = [
            widget.value_from_datadict(data, files, name + '_%s' % i)
            for i, widget in enumerate(self.widgets)]
        try:
            D = date(day=1, month=int(datelist[1]),
                     year=int(datelist[0]))
        except ValueError:
            return ''
        else:
            return str(D)



import datetime

from django.db import models
from django.core import exceptions
from django.utils.translation import ugettext_lazy as _


from . import forms
from . import widgets
from . import Month
from .util import string_type


class MonthField(models.DateField):
    description = "A specific month of a specific year."
    widget = widgets.MonthSelectorWidget

    default_error_messages = {
        'invalid_year': _("Year informed invalid. Enter at least 4 digits."),
    }

    def to_python(self, value):
        if isinstance(value, Month):
            month = value
        elif isinstance(value, datetime.date):
            month = Month.from_date(value)
            if len(str(month.year)) < 4:
                raise exceptions.ValidationError(
                    self.error_messages['invalid_year'],
                    code='invalid_year',
                    params={'value': value},
                )
        elif isinstance(value, string_type):
            month = Month.from_string(value)
        else:
            month = None
        return month

    def get_prep_value(self, value):
        month = self.to_python(value)
        if month is not None:
            return month.first_day()
        return None

    def from_db_value(self, value, expression, connection):
        return self.to_python(value)

    def clean(self, value, instance):
        return self.to_python(value)

    def formfield(self, **kwargs):
        # The widget is allready being specified somewhere by models.DateField...
        kwargs['widget'] = self.widget
        defaults = {
            'form_class': forms.MonthField
        }
        defaults.update(kwargs)
        return super(MonthField, self).formfield(**defaults)



# Put your test models here

from django.db import models

from django_dj_plugin.month.models import MonthField


# Create your models here.

class Example(models.Model):
    name = models.CharField(max_length=20, blank=True)
    month = MonthField("Month Value", help_text="some help...")

    def __str__(self):
        return str(self.month)



import datetime

from .util import string_type


def days(days):
    return datetime.timedelta(days=days)


class Month(object):
    def __init__(self, year, month):
        self.year = year
        self.month = month
        self._date = datetime.date(year=self.year, month=self.month, day=1)

    @classmethod
    def from_int(cls, months):
        y, m = divmod(months, 12)
        m += 1
        return cls(y, m)

    @classmethod
    def from_date(cls, date):
        return cls(date.year, date.month)

    @classmethod
    def from_string(cls, date):
        y = int(date[:4])
        m = int(date[5:7])
        return cls(y, m)

    def __add__(self, x):
        '''x is an integer'''
        return Month.from_int(int(self) + x)

    def __sub__(self, x):
        '''x is integer or Month instance'''
        if isinstance(x, Month):
            return int(self) - int(x)
        else:
            return Month.from_int(int(self) - int(x))

    def next_month(self):
        return self + 1

    def prev_month(self):
        return self - 1

    def first_day(self):
        return self._date

    def last_day(self):
        return self.next_month().first_day() - days(1)

    def __int__(self):
        return self.year * 12 + self.month - 1

    def __contains__(self, date):
        return self == date

    def __eq__(self, x):
        if isinstance(x, Month):
            return x.month == self.month and x.year == self.year
        if isinstance(x, datetime.date):
            return self.year == x.year and self.month == x.month
        if isinstance(x, int):
            return x == int(self)
        if isinstance(x, string_type):
            return str(self) == x[:7]

    def __gt__(self, x):
        if isinstance(x, Month):
            if self.year != x.year: return self.year > x.year
            return self.month > x.month
        if isinstance(x, datetime.date):
            return self.first_day() > x
        if isinstance(x, int):
            return int(self) > x
        if isinstance(x, string_type):
            return str(self) > x[:7]

    def __ne__(self, x):
        return not self == x

    def __le__(self, x):
        return not self > x

    def __ge__(self, x):
        return (self > x) or (self == x)

    def __lt__(self, x):
        return not self >= x

    def __str__(self):
        return '%s-%02d' % (self.year, self.month)

    def __unicode__(self):
        return self.__str__()

    def __repr__(self):
        return self.__str__()

    def __hash__(self):
        return hash(self.datestring())

    def datestring(self):
        return self.first_day().isoformat()

    isoformat = datestring

    def range(self, x):
        '''x must be an instance of Month that is larger than self.
        returns a list of Month objects that make up the timespan from self to x (inclusive)'''
        months_as_ints = range(int(self), int(x) + 1)
        return [Month.from_int(i) for i in months_as_ints]

    def strftime(self, fmt):
        return self._date.strftime(fmt)



# coding: utf-8
from django.http import HttpRequest


def get_real_ip(request: HttpRequest):
    """
    获取用户真实 IP

    :param request:
    :return:
    """
    real_ip = request.META.get('HTTP_X_REAL_IP', None) or request.META['REMOTE_ADDR']
    return real_ip



# Generated by Django 2.2.9 on 2020-01-22 06:24

from django.db import migrations, models
import django_dj_plugin.month.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Example',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=20)),
                ('month', django_dj_plugin.month.models.MonthField(help_text='some help...', verbose_name='Month Value')),
            ],
        ),
    ]
